@@ -1,4 +1,4 @@
-binomial.CARsepspatial <- function(formula, data=NULL, trials, W, burnin, n.sample, thin=1,  prior.mean.beta=NULL, prior.var.beta=NULL, prior.tau2=NULL, fix.rho.S=FALSE, rho.S=NULL, fix.rho.T=FALSE, rho.T=NULL, MALA=TRUE, verbose=TRUE)
+binomial.CARsepspatial <- function(formula, data=NULL, trials, W, burnin, n.sample, thin=1,  prior.mean.beta=NULL, prior.var.beta=NULL, prior.tau2=NULL, rho.S=NULL, rho.T=NULL, MALA=TRUE, verbose=TRUE)
 {
   ##############################################
   #### Format the arguments and check for errors
@@ -18,10 +18,8 @@ binomial.CARsepspatial <- function(formula, data=NULL, trials, W, burnin, n.samp
   offset <- frame.results$offset
   Y <- frame.results$Y
   failures <- trials - Y
-  Y.miss <- frame.results$Y.miss
-  which.miss <- frame.results$which.miss
   n.miss <- frame.results$n.miss  
-
+    if(n.miss>0) stop("the response has missing 'NA' values.", call.=FALSE)
   #### Check on MALA argument
     if(length(MALA)!=1) stop("MALA is not length 1.", call.=FALSE)
     if(!is.logical(MALA)) stop("MALA is not logical.", call.=FALSE) 
@@ -32,38 +30,37 @@ binomial.CARsepspatial <- function(formula, data=NULL, trials, W, burnin, n.samp
   if(min(trials)<=0) stop("the numbers of trials has zero or negative values.", call.=FALSE)
   if(sum(Y>trials)>0) stop("the response variable has larger values that the numbers of trials.", call.=FALSE)
   
-  ## Check for errors on rho and fix.rho
-  if(!is.logical(fix.rho.S)) stop("fix.rho.S is not logical.", call.=FALSE)   
-  if(fix.rho.S & is.null(rho.S)) stop("rho.S is fixed but an initial value was not set.", call.=FALSE)   
-  if(fix.rho.S & !is.numeric(rho.S) ) stop("rho.S is not numeric.", call.=FALSE)  
-  if(!is.logical(fix.rho.T)) stop("fix.rho.T is not logical.", call.=FALSE)   
-  if(fix.rho.T & is.null(rho.T)) stop("rho.T is fixed but an initial value was not set.", call.=FALSE)   
-  if(fix.rho.T & !is.numeric(rho.T) ) stop("rho.T is not numeric.", call.=FALSE)    
   
-  
-  if(fix.rho.S)
-  {
+#### Check on the rho arguments
+    if(is.null(rho.S))
+    {
+    rho <- runif(1)
+    fix.rho.S <- FALSE   
+    }else
+    {
     rho <- rho.S
-  }else
-  {
-    rho <- runif(1)       
-  }
+    fix.rho.S <- TRUE
+    }  
+    if(!is.numeric(rho)) stop("rho.S is fixed but is not numeric.", call.=FALSE)  
+    if(rho<0 ) stop("rho.S is outside the range [0, 1].", call.=FALSE)  
+    if(rho>1 ) stop("rho.S is outside the range [0, 1].", call.=FALSE)    
   
-  if(fix.rho.T)
-  {
+    if(is.null(rho.T))
+    {
+    lambda <- runif(1)
+    fix.rho.T <- FALSE   
+    }else
+    {
     lambda <- rho.T
-  }else
-  {
-    lambda <- runif(1)       
-  }   
-  
-  if(rho<0 ) stop("rho.S is outside the range [0, 1].", call.=FALSE)  
-  if(rho>1 ) stop("rho.S is outside the range [0, 1].", call.=FALSE)  
-  if(lambda<0 ) stop("rho.T is outside the range [0, 1].", call.=FALSE)  
-  if(lambda>1 ) stop("rho.T is outside the range [0, 1].", call.=FALSE) 
+    fix.rho.T <- TRUE
+    }
+    if(!is.numeric(lambda)) stop("rho.T is fixed but is not numeric.", call.=FALSE)  
+    if(lambda<0 ) stop("rho.T is outside the range [0, 1].", call.=FALSE)  
+    if(lambda>1 ) stop("rho.T is outside the range [0, 1].", call.=FALSE)  
+
   
   #### CAR quantities
-  W.quants <- common.Wcheckformat.leroux(W, fix.rho.S, rho)
+  W.quants <- common.Wcheckformat.leroux(W)
   K <- W.quants$n
   N <- N.all / K
   W <- W.quants$W
@@ -97,7 +94,7 @@ binomial.CARsepspatial <- function(formula, data=NULL, trials, W, burnin, n.samp
    #### Check and specify the priors
   ## Put in default priors
   if(is.null(prior.mean.beta)) prior.mean.beta <- rep(0, p)
-  if(is.null(prior.var.beta)) prior.var.beta <- rep(1000, p)
+  if(is.null(prior.var.beta)) prior.var.beta <- rep(100000, p)
   if(is.null(prior.tau2)) prior.tau2 <- c(1, 0.01)
   
   prior.beta.check(prior.mean.beta, prior.var.beta, p)
@@ -128,8 +125,7 @@ binomial.CARsepspatial <- function(formula, data=NULL, trials, W, burnin, n.samp
   if(!fix.rho.T) samples.lambda <- array(NA, c(n.keep, 1))
   samples.delta <- array(NA, c(n.keep, N))     
   samples.fitted <- array(NA, c(n.keep, N.all))
-  samples.deviance <- array(NA, c(n.keep, 1))
-  samples.like <- array(NA, c(n.keep, N.all))
+  samples.loglike <- array(NA, c(n.keep, N.all))
   
   #### Specify the Metropolis quantities
   accept.all <- rep(0,10)
@@ -206,13 +202,8 @@ binomial.CARsepspatial <- function(formula, data=NULL, trials, W, burnin, n.samp
   offset.mat <- matrix(offset, nrow=K, ncol=N, byrow=FALSE) 
   regression.mat <- matrix(X.standardised %*% beta, nrow=K, ncol=N, byrow=FALSE)
   Y.mat <- matrix(Y, nrow=K, ncol=N, byrow=FALSE)
-  Y.mat.trans <- t(Y.mat)
-  which.miss.mat <- matrix(which.miss, nrow=K, ncol=N, byrow=FALSE)
-  which.miss.mat.trans <- t(which.miss.mat)
   trials.mat <- matrix(trials, nrow=K, ncol=N, byrow=FALSE)
-  trials.mat.trans <- t(trials.mat)
   failures.mat <- matrix(failures, nrow=K, ncol=N, byrow=FALSE)
-  failures.mat.trans <- t(failures.mat)
   delta.mat <- matrix(delta, nrow=K, ncol=N, byrow=TRUE)
   
   ## Check for islands
@@ -231,7 +222,7 @@ binomial.CARsepspatial <- function(formula, data=NULL, trials, W, burnin, n.samp
   ## Start timer
   if(verbose)
   {
-    cat("Generating", n.keep, "post burnin and thinned (if requested) samples\n", sep = " ")
+    cat("Generating", n.keep, "post burnin and thinned (if requested) samples.\n", sep = " ")
     progressBar <- txtProgressBar(style = 3)
     percentage.points<-round((1:100/100)*n.sample)
   }else
@@ -247,10 +238,10 @@ binomial.CARsepspatial <- function(formula, data=NULL, trials, W, burnin, n.samp
     offset.temp <- as.numeric(offset.mat + phi.mat + delta.mat)    
     if(p>2)
     {
-      temp <- binomialbetaupdateMALA(X.standardised, N.all, p, beta, offset.temp, Y.miss, failures, trials, prior.mean.beta, prior.var.beta, which.miss, n.beta.block, proposal.sd.beta, list.block)
+      temp <- binomialbetaupdateMALA(X.standardised, N.all, p, beta, offset.temp, Y, failures, trials, prior.mean.beta, prior.var.beta, n.beta.block, proposal.sd.beta, list.block)
     }else
     {
-      temp <- binomialbetaupdateRW(X.standardised, N.all, p, beta, offset.temp, Y.miss, failures, prior.mean.beta, prior.var.beta, which.miss, proposal.sd.beta)
+      temp <- binomialbetaupdateRW(X.standardised, N.all, p, beta, offset.temp, Y, failures, prior.mean.beta, prior.var.beta, proposal.sd.beta)
     }
     beta <- temp[[1]]
     accept[1] <- accept[1] + temp[[2]]
@@ -290,10 +281,10 @@ binomial.CARsepspatial <- function(formula, data=NULL, trials, W, burnin, n.samp
     delta.offset <- t(offset.mat + phi.mat + regression.mat)
     if(MALA==TRUE)
     {
-      temp2 <- binomialcarupdateMALA(D.triplet, D.begfin, D.triplet.sum, N, delta, sig2, Y.mat.trans, failures.mat.trans, trials.mat.trans, proposal.sd.delta, lambda, delta.offset, K, rep(1,K), which.miss.mat.trans)
+      temp2 <- binomialcarupdateMALA(D.triplet, D.begfin, D.triplet.sum, N, delta, sig2, t(Y.mat), t(failures.mat), t(trials.mat), proposal.sd.delta, lambda, delta.offset, K, rep(1,K))
     }else
     {
-      temp2 <- binomialcarupdateRW(D.triplet, D.begfin, D.triplet.sum, N, delta, sig2, Y.mat.trans, failures.mat.trans, proposal.sd.delta, lambda, delta.offset, K, rep(1,K), which.miss.mat.trans)
+      temp2 <- binomialcarupdateRW(D.triplet, D.begfin, D.triplet.sum, N, delta, sig2, t(Y.mat), t(failures.mat), proposal.sd.delta, lambda, delta.offset, K, rep(1,K))
     }
     delta <- temp2[[1]]
     delta <- delta - mean(delta)
@@ -369,9 +360,9 @@ binomial.CARsepspatial <- function(formula, data=NULL, trials, W, burnin, n.samp
     lp <- as.numeric(offset.mat + regression.mat + phi.mat + delta.mat)
     prob <- exp(lp) / (1+exp(lp))
     fitted <- trials * prob
-    deviance.all <- dbinom(x=Y, size=trials, prob=prob, log=TRUE)
-    like <- exp(deviance.all)
-    deviance <- -2 * sum(deviance.all, na.rm=TRUE)  
+    loglike <- dbinom(x=Y, size=trials, prob=prob, log=TRUE)
+
+
     
     ###################
     ## Save the results
@@ -386,9 +377,8 @@ binomial.CARsepspatial <- function(formula, data=NULL, trials, W, burnin, n.samp
       samples.tau2[ele, ] <- tau2
       samples.sig2[ele, ] <- sig2
       samples.delta[ele, ] <- delta
-      samples.deviance[ele, ] <- deviance
       samples.fitted[ele, ] <- fitted
-      samples.like[ele, ] <- like
+      samples.loglike[ele, ] <- loglike
     }else
     {
     }
@@ -428,70 +418,59 @@ binomial.CARsepspatial <- function(formula, data=NULL, trials, W, burnin, n.samp
   # end timer
   if(verbose)
   {
-    cat("\nSummarising results")
+      cat("\nSummarising results.")
     close(progressBar)
   }else
   {}
   
-  ###################################
-  #### Summarise and save the results 
-  ###################################
-  ## Compute the acceptance rates
-  accept.beta <- 100 * accept.all[1] / accept.all[2]
-  accept.phi <- 100 * accept.all[3] / accept.all[4]
-  accept.delta <- 100 * accept.all[7] / accept.all[8]
   
-  if(!fix.rho.S)
-  {
-    accept.rho <- 100 * accept.all[5] / accept.all[6]
-  }else
-  {
-    accept.rho <- NA    
-  }
+###################################
+#### Summarise and save the results 
+###################################
+## Compute the acceptance rates
+accept.beta <- 100 * accept.all[1] / accept.all[2]
+accept.phi <- 100 * accept.all[3] / accept.all[4]
+accept.delta <- 100 * accept.all[7] / accept.all[8]
+   if(!fix.rho.S)
+   {
+   accept.rho <- 100 * accept.all[5] / accept.all[6]
+   }else
+   {
+   accept.rho <- NA    
+   }
   
-  if(!fix.rho.T)
-  {
-    accept.lambda <- 100 * accept.all[9] / accept.all[10]
-  }else
-  {
-    accept.lambda <- NA    
-  }
+   if(!fix.rho.T)
+   {
+   accept.lambda <- 100 * accept.all[9] / accept.all[10]
+   }else
+   {
+   accept.lambda <- NA    
+   }
+accept.final <- c(accept.beta, accept.phi, accept.delta, accept.rho, accept.lambda)
+names(accept.final) <- c("beta", "phi", "delta", "rho.S", "rho.T")
   
-  accept.final <- c(accept.beta, accept.phi, accept.delta, accept.rho, accept.lambda)
-  names(accept.final) <- c("beta", "phi", "delta", "rho.S", "rho.T")
   
-  ## Compute information criterion (DIC, DIC3, WAIC)
-  median.beta <- apply(samples.beta,2,median)
-  regression.mat <- matrix(X.standardised %*% median.beta, nrow=K, ncol=N, byrow=FALSE)   
-  median.phi <- matrix(apply(samples.phi, 2, median), nrow=K, ncol=N)
-  median.delta <- apply(samples.delta,2,median)
-  delta.mat <- matrix(median.delta, nrow=K, ncol=N, byrow=TRUE)
-  lp.median <- as.numeric(offset.mat + median.phi + regression.mat + delta.mat)   
-  median.prob <- exp(lp.median)  / (1 + exp(lp.median))
-  fitted.median <- trials * median.prob
-  deviance.fitted <- -2 * sum(dbinom(x=Y, size=trials, prob=median.prob, log=TRUE), na.rm = T)
-  p.d <- median(samples.deviance) - deviance.fitted
-  DIC <- 2 * median(samples.deviance) - deviance.fitted     
+#### Compute the fitted deviance
+mean.beta <- apply(samples.beta,2,mean)
+regression.mat <- matrix(X.standardised %*% mean.beta, nrow=K, ncol=N, byrow=FALSE)   
+mean.phi <- matrix(apply(samples.phi, 2, mean), nrow=K, ncol=N)
+mean.delta <- apply(samples.delta,2,mean)
+delta.mat <- matrix(mean.delta, nrow=K, ncol=N, byrow=TRUE)
+lp.mean <- as.numeric(offset.mat + mean.phi + regression.mat + delta.mat)   
+mean.prob <- exp(lp.mean)  / (1 + exp(lp.mean))
+deviance.fitted <- -2 * sum(dbinom(x=Y, size=trials, prob=mean.prob, log=TRUE), na.rm = T)
+
+    
+#### Model fit criteria
+modelfit <- common.modelfit(samples.loglike, deviance.fitted)
   
-  #### Watanabe-Akaike Information Criterion (WAIC)
-  LPPD <- sum(log(apply(samples.like,2,mean)), na.rm=TRUE)
-  p.w <- sum(apply(log(samples.like),2,var), na.rm=TRUE)
-  WAIC <- -2 * (LPPD - p.w)
   
-  ## Compute the LMPL
-  CPO <- rep(NA, N.all)
-  for(j in 1:N.all)
-  {
-    CPO[j] <- 1/median((1 / dbinom(x=Y[j], size=trials[j], prob=(samples.fitted[ ,j] / trials[j]))))    
-  }
-  LMPL <- sum(log(CPO), na.rm=TRUE)  
-  
-  ## Create the fitted values and residuals
-  fitted.values <- apply(samples.fitted, 2, median)
-  response.residuals <- as.numeric(Y) - fitted.values
-  pearson.residuals <- response.residuals /sqrt(fitted.values * (1 - median.prob))
-  deviance.residuals <- sign(response.residuals) * sqrt(2 * (Y * log(Y/fitted.values) + (trials-Y) * log((trials-Y)/(trials - fitted.values))))
-  residuals <- data.frame(response=response.residuals, pearson=pearson.residuals, deviance=deviance.residuals)
+#### Create the fitted values and residuals
+fitted.values <- apply(samples.fitted, 2, mean)
+response.residuals <- as.numeric(Y) - fitted.values
+pearson.residuals <- response.residuals /sqrt(fitted.values * (1 - mean.prob))
+residuals <- data.frame(response=response.residuals, pearson=pearson.residuals)
+
   
   #### Transform the parameters back to the origianl covariate scale.
   samples.beta.orig <- common.betatransform(samples.beta, X.indicator, X.mean, X.sd, p, FALSE)
@@ -536,11 +515,8 @@ binomial.CARsepspatial <- function(formula, data=NULL, trials, W, burnin, n.samp
   summary.results[ , 1:3] <- round(summary.results[ , 1:3], 4)
   summary.results[ , 4:7] <- round(summary.results[ , 4:7], 1)
   
-  ## Compile and return the results
-  loglike <- (-0.5 * deviance.fitted)
-  modelfit <- c(DIC, p.d, WAIC, p.w, LMPL, loglike)
-  names(modelfit) <- c("DIC", "p.d", "WAIC", "p.w", "LMPL", "loglikelihood")
   
+#### Compile and return the results
   if(fix.rho.S & fix.rho.T)
   {
     samples.rhoext <- NA
@@ -566,7 +542,7 @@ binomial.CARsepspatial <- function(formula, data=NULL, trials, W, burnin, n.samp
   if(verbose)
   {
     b<-proc.time()
-    cat(" finished in ", round(b[3]-a[3], 1), "seconds")
+    cat("Finished in ", round(b[3]-a[3], 1), "seconds.\n")
   }else
   {}
   return(results)
